@@ -1,4 +1,4 @@
-/*	$OpenBSD: bgpd.h,v 1.378 2019/03/31 16:57:38 claudio Exp $ */
+/*	$OpenBSD: bgpd.h,v 1.394 2019/10/01 08:57:47 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
@@ -36,16 +36,17 @@
 #define	BGP_VERSION			4
 #define	BGP_PORT			179
 #define	PEER_DESCR_LEN			32
-#define	SHUT_COMM_LEN			129
+#define	SHUT_COMM_LEN			256	/* includes NUL terminator */
 #define	PFTABLE_LEN			32
 #define	TCP_MD5_KEY_LEN			80
 #define	IPSEC_ENC_KEY_LEN		32
 #define	IPSEC_AUTH_KEY_LEN		20
-#define	SET_NAME_LEN			64
+#define	SET_NAME_LEN			128
 
 #define	MAX_PKTSIZE			4096
 #define	MIN_HOLDTIME			3
 #define	READ_BUF_SIZE			65535
+#define	MAX_SOCK_BUF			(4 * READ_BUF_SIZE)
 #define	RT_BUF_SIZE			16384
 #define	MAX_RTSOCK_BUF			(2 * 1024 * 1024)
 #define	MAX_COMM_MATCH			3
@@ -55,10 +56,9 @@
 #define	BGPD_OPT_NOACTION		0x0004
 #define	BGPD_OPT_FORCE_DEMOTE		0x0008
 
-#define	BGPD_FLAG_NO_EVALUATE		0x0002
 #define	BGPD_FLAG_REFLECTOR		0x0004
-#define	BGPD_FLAG_NEXTHOP_BGP		0x0080
-#define	BGPD_FLAG_NEXTHOP_DEFAULT	0x1000
+#define	BGPD_FLAG_NEXTHOP_BGP		0x0010
+#define	BGPD_FLAG_NEXTHOP_DEFAULT	0x0020
 #define	BGPD_FLAG_DECISION_MASK		0x0f00
 #define	BGPD_FLAG_DECISION_ROUTEAGE	0x0100
 #define	BGPD_FLAG_DECISION_TRANS_AS	0x0200
@@ -90,17 +90,19 @@
 #define	F_REJECT		0x0080
 #define	F_BLACKHOLE		0x0100
 #define	F_LONGER		0x0200
-#define	F_MPLS			0x0400
-#define	F_CTL_DETAIL		0x1000	/* only used by bgpctl */
-#define	F_CTL_ADJ_IN		0x2000
-#define	F_CTL_ADJ_OUT		0x4000
+#define	F_SHORTER		0x0400
+#define	F_MPLS			0x0800
+#define	F_CTL_DETAIL		0x1000	/* only set on requests */
+#define	F_CTL_ADJ_IN		0x2000	/* only set on requests */
+#define	F_CTL_ADJ_OUT		0x4000	/* only set on requests */
 #define	F_CTL_ACTIVE		0x8000
 #define	F_RTLABEL		0x10000
 #define	F_CTL_SSV		0x20000	/* only used by bgpctl */
-#define	F_CTL_INVALID		0x40000 /* only used by bgpctl */
+#define	F_CTL_INVALID		0x40000 /* only set on requests */
 #define	F_CTL_OVS_VALID		0x80000
 #define	F_CTL_OVS_INVALID	0x100000
 #define	F_CTL_OVS_NOTFOUND	0x200000
+#define	F_CTL_NEIGHBORS		0x400000 /* only used by bgpctl */
 
 /*
  * Note that these numeric assignments differ from the numbers commonly
@@ -119,8 +121,8 @@
  * IMSG_XON message will be sent and the RDE will produce more messages again.
  */
 #define RDE_RUNNER_ROUNDS	100
-#define SESS_MSG_HIGH_MARK	300
-#define SESS_MSG_LOW_MARK	50
+#define SESS_MSG_HIGH_MARK	2000
+#define SESS_MSG_LOW_MARK	500
 #define CTL_MSG_HIGH_MARK	500
 #define CTL_MSG_LOW_MARK	100
 
@@ -242,7 +244,7 @@ TAILQ_HEAD(listen_addrs, listen_addr);
 TAILQ_HEAD(filter_set_head, filter_set);
 
 struct peer;
-TAILQ_HEAD(peer_head, peer);
+RB_HEAD(peer_head, peer);
 
 struct l3vpn;
 SIMPLEQ_HEAD(l3vpn_head, l3vpn);
@@ -292,7 +294,7 @@ struct bgpd_config {
 	struct rde_prefixset_head		 rde_prefixsets;
 	struct rde_prefixset_head		 rde_originsets;
 	struct rde_prefixset			 rde_roa;
-	struct as_set_head			*as_sets;
+	struct as_set_head			 as_sets;
 	char					*csock;
 	char					*rcsock;
 	int					 flags;
@@ -406,6 +408,12 @@ struct peer_config {
 	u_int8_t		 flags;
 };
 
+#define	PEER_ID_NONE		0
+#define	PEER_ID_SELF		1
+#define	PEER_ID_STATIC_MIN	2	/* exclude self */
+#define	PEER_ID_STATIC_MAX	(UINT_MAX / 2)
+#define	PEER_ID_DYN_MAX		UINT_MAX
+
 #define PEERFLAG_TRANS_AS	0x01
 #define PEERFLAG_LOG_UPDATES	0x02
 
@@ -422,7 +430,6 @@ enum network_type {
 struct network_config {
 	struct bgpd_addr	 prefix;
 	struct filter_set_head	 attrset;
-	struct rde_aspath	*asp;
 	char			 psname[SET_NAME_LEN];
 	u_int64_t		 rd;
 	u_int16_t		 rtlabel;
@@ -456,6 +463,7 @@ enum imsg_type {
 	IMSG_CTL_SHOW_INTERFACE,
 	IMSG_CTL_SHOW_RIB,
 	IMSG_CTL_SHOW_RIB_PREFIX,
+	IMSG_CTL_SHOW_RIB_COMMUNITIES,
 	IMSG_CTL_SHOW_RIB_ATTR,
 	IMSG_CTL_SHOW_NETWORK,
 	IMSG_CTL_SHOW_RIB_MEM,
@@ -508,6 +516,7 @@ enum imsg_type {
 	IMSG_MRT_CLOSE,
 	IMSG_KROUTE_CHANGE,
 	IMSG_KROUTE_DELETE,
+	IMSG_KROUTE_FLUSH,
 	IMSG_NEXTHOP_ADD,
 	IMSG_NEXTHOP_REMOVE,
 	IMSG_NEXTHOP_UPDATE,
@@ -780,28 +789,21 @@ struct filter_ovs {
 	u_int8_t		 is_set;
 };
 
-struct filter_community {
-	u_int8_t	type;
-	u_int8_t	dflag1;	/* one of set, any, local-as, neighbor-as */
-	u_int8_t	dflag2;
-	u_int8_t	dflag3;
-	union {
-		struct basic {
-			u_int32_t	data1;
-			u_int32_t	data2;
-		} b;
-		struct large {
-			u_int32_t	data1;
-			u_int32_t	data2;
-			u_int32_t	data3;
-		} l;
-		struct ext {
-			u_int32_t	data1;
-			u_int64_t	data2;
-			short		type;
-			u_int8_t	subtype;	/* if extended type */
-		} e;
-	}		c;
+/*
+ * Communities are encoded depending on their type. The low byte of flags
+ * is the COMMUNITY_TYPE (BASIC, LARGE, EXT). BASIC encoding is just using
+ * data1 and data2, LARGE uses all data fields and EXT is also using all
+ * data fields. The 4-byte flags fields consists of up to 3 data flags
+ * for e.g. COMMUNITY_ANY and the low byte is the community type.
+ * If flags is 0 the community struct is unused. If the upper 24bit of
+ * flags is 0 a fast compare can be used.
+ * The code uses a type cast to u_int8_t to access the type.
+ */
+struct community {
+	u_int32_t	flags;
+	u_int32_t	data1;
+	u_int32_t	data2;
+	u_int32_t	data3;
 };
 
 struct ctl_show_rib_request {
@@ -809,7 +811,7 @@ struct ctl_show_rib_request {
 	struct ctl_neighbor	neighbor;
 	struct bgpd_addr	prefix;
 	struct filter_as	as;
-	struct filter_community community;
+	struct community	community;
 	u_int32_t		flags;
 	u_int8_t		validation_state;
 	pid_t			pid;
@@ -857,11 +859,11 @@ struct filter_peers {
 	u_int8_t	ibgp;
 };
 
-/* special community type */
+/* special community type, keep in sync with the attribute type */
 #define	COMMUNITY_TYPE_NONE		0
-#define	COMMUNITY_TYPE_BASIC		1
-#define	COMMUNITY_TYPE_EXT		2
-#define	COMMUNITY_TYPE_LARGE		3
+#define	COMMUNITY_TYPE_BASIC		8
+#define	COMMUNITY_TYPE_EXT		16
+#define	COMMUNITY_TYPE_LARGE		32
 
 #define	COMMUNITY_ANY			1
 #define	COMMUNITY_NEIGHBOR_AS		2
@@ -878,7 +880,7 @@ struct filter_peers {
 
 /* extended community definitions */
 #define EXT_COMMUNITY_IANA		0x80
-#define EXT_COMMUNITY_TRANSITIVE	0x40
+#define EXT_COMMUNITY_NON_TRANSITIVE	0x40
 #define EXT_COMMUNITY_VALUE		0x3f
 /* extended types transitive */
 #define EXT_COMMUNITY_TRANS_TWO_AS	0x00	/* 2 octet AS specific */
@@ -964,7 +966,7 @@ struct filter_match {
 	struct filter_nexthop		nexthop;
 	struct filter_as		as;
 	struct filter_aslen		aslen;
-	struct filter_community		community[MAX_COMM_MATCH];
+	struct community		community[MAX_COMM_MATCH];
 	struct filter_prefixset		prefixset;
 	struct filter_originset		originset;
 	struct filter_ovs		ovs;
@@ -1021,7 +1023,7 @@ struct filter_set {
 		int32_t				 relative;
 		struct bgpd_addr		 nexthop;
 		struct nexthop			*nh;
-		struct filter_community		 community;
+		struct community		 community;
 		char				 pftable[PFTABLE_LEN];
 		char				 rtlabel[RTLABEL_LEN];
 		u_int8_t			 origin;
@@ -1082,7 +1084,6 @@ extern struct rib_names ribnames;
 #define F_RIB_NOEVALUATE	0x0002
 #define F_RIB_NOFIB		0x0004
 #define F_RIB_NOFIBSYNC		0x0008
-#define F_RIB_HASNOFIB		(F_RIB_NOFIB | F_RIB_NOEVALUATE)
 
 /* 4-byte magic AS number */
 #define AS_TRANS	23456
@@ -1099,6 +1100,10 @@ struct rde_memstats {
 	long long	aspath_cnt;
 	long long	aspath_size;
 	long long	aspath_refs;
+	long long	comm_cnt;
+	long long	comm_nmemb;
+	long long	comm_size;
+	long long	comm_refs;
 	long long	attr_cnt;
 	long long	attr_refs;
 	long long	attr_data;
@@ -1175,6 +1180,7 @@ int	control_imsg_relay(struct imsg *);
 /* config.c */
 struct bgpd_config	*new_config(void);
 void		copy_config(struct bgpd_config *, struct bgpd_config *);
+void		free_l3vpns(struct l3vpn_head *);
 void		free_config(struct bgpd_config *);
 void		free_prefixsets(struct prefixset_head *);
 void		free_rde_prefixsets(struct rde_prefixset_head *);
@@ -1189,13 +1195,14 @@ int		prefixset_cmp(struct prefixset_item *, struct prefixset_item *);
 RB_PROTOTYPE(prefixset_tree, prefixset_item, entry, prefixset_cmp);
 
 /* kroute.c */
-int		 kr_init(void);
+int		 kr_init(int *);
 int		 ktable_update(u_int, char *, int, u_int8_t);
 void		 ktable_preload(void);
 void		 ktable_postload(u_int8_t);
 int		 ktable_exists(u_int, u_int *);
 int		 kr_change(u_int, struct kroute_full *,  u_int8_t);
 int		 kr_delete(u_int, struct kroute_full *, u_int8_t);
+int		 kr_flush(u_int);
 void		 kr_shutdown(u_int8_t, u_int);
 void		 kr_fib_couple(u_int, u_int8_t);
 void		 kr_fib_couple_all(u_int8_t);
@@ -1275,7 +1282,7 @@ void		 as_sets_mark_dirty(struct as_set_head *, struct as_set_head *);
 int		 as_set_match(const struct as_set *, u_int32_t);
 
 struct set_table	*set_new(size_t, size_t);
-void		 	 set_free(struct set_table *);
+void			 set_free(struct set_table *);
 int			 set_add(struct set_table *, void *, size_t);
 void			*set_get(struct set_table *, size_t *);
 void			 set_prep(struct set_table *);
