@@ -1,4 +1,4 @@
-/*	$OpenBSD: printconf.c,v 1.134 2019/03/31 16:57:38 claudio Exp $	*/
+/*	$OpenBSD: printconf.c,v 1.140 2019/08/07 10:26:41 claudio Exp $	*/
 
 /*
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
@@ -24,14 +24,13 @@
 #include <string.h>
 
 #include "bgpd.h"
-#include "mrt.h"
 #include "session.h"
 #include "rde.h"
 #include "log.h"
 
 void		 print_prefix(struct filter_prefix *p);
-const char	*community_type(struct filter_community *c);
-void		 print_community(struct filter_community *c);
+const char	*community_type(struct community *c);
+void		 print_community(struct community *c);
 void		 print_origin(u_int8_t);
 void		 print_set(struct filter_set_head *);
 void		 print_mainconf(struct bgpd_config *);
@@ -105,9 +104,9 @@ print_prefix(struct filter_prefix *p)
 }
 
 const char *
-community_type(struct filter_community *c)
+community_type(struct community *c)
 {
-	switch (c->type) {
+	switch ((u_int8_t)c->flags) {
 	case COMMUNITY_TYPE_BASIC:
 		return "community";
 	case COMMUNITY_TYPE_LARGE:
@@ -120,13 +119,15 @@ community_type(struct filter_community *c)
 }
 
 void
-print_community(struct filter_community *c)
+print_community(struct community *c)
 {
 	struct in_addr addr;
+	short type;
+	u_int8_t subtype;
 
-	switch (c->type) {
+	switch ((u_int8_t)c->flags) {
 	case COMMUNITY_TYPE_BASIC:
-		switch (c->dflag1) {
+		switch ((c->flags >> 8) & 0xff) {
 		case COMMUNITY_ANY:
 			printf("*:");
 			break;
@@ -137,10 +138,10 @@ print_community(struct filter_community *c)
 			printf("local-as:");
 			break;
 		default:
-			printf("%u:", c->c.b.data1);
+			printf("%u:", c->data1);
 			break;
 		}
-		switch (c->dflag2) {
+		switch ((c->flags >> 16) & 0xff) {
 		case COMMUNITY_ANY:
 			printf("* ");
 			break;
@@ -151,12 +152,12 @@ print_community(struct filter_community *c)
 			printf("local-as ");
 			break;
 		default:
-			printf("%u ", c->c.b.data2);
+			printf("%u ", c->data2);
 			break;
 		}
 		break;
 	case COMMUNITY_TYPE_LARGE:
-		switch (c->dflag1) {
+		switch ((c->flags >> 8) & 0xff) {
 		case COMMUNITY_ANY:
 			printf("*:");
 			break;
@@ -167,10 +168,10 @@ print_community(struct filter_community *c)
 			printf("local-as:");
 			break;
 		default:
-			printf("%u:", c->c.l.data1);
+			printf("%u:", c->data1);
 			break;
 		}
-		switch (c->dflag2) {
+		switch ((c->flags >> 16) & 0xff) {
 		case COMMUNITY_ANY:
 			printf("*:");
 			break;
@@ -181,10 +182,10 @@ print_community(struct filter_community *c)
 			printf("local-as:");
 			break;
 		default:
-			printf("%u:", c->c.l.data2);
+			printf("%u:", c->data2);
 			break;
 		}
-		switch (c->dflag3) {
+		switch ((c->flags >> 24) & 0xff) {
 		case COMMUNITY_ANY:
 			printf("* ");
 			break;
@@ -195,58 +196,56 @@ print_community(struct filter_community *c)
 			printf("local-as ");
 			break;
 		default:
-			printf("%u ", c->c.l.data3);
+			printf("%u ", c->data3);
 			break;
 		}
 		break;
 	case COMMUNITY_TYPE_EXT:
-		if (c->dflag3 == COMMUNITY_ANY) {
+		if ((c->flags >> 24 & 0xff) == COMMUNITY_ANY) {
 			printf("* * ");
 			break;
 		}
-		printf("%s ", log_ext_subtype(c->c.e.type, c->c.e.subtype));
-		if (c->dflag1 == COMMUNITY_ANY) {
+		type = (int32_t)c->data3 >> 8;
+		subtype = c->data3;
+		printf("%s ", log_ext_subtype(type, subtype));
+		if ((c->flags >> 8 & 0xff) == COMMUNITY_ANY) {
 			printf("* ");
 			break;
 		}
 
-		switch (c->c.e.type) {
+		switch (type) {
 		case EXT_COMMUNITY_TRANS_TWO_AS:
 		case EXT_COMMUNITY_TRANS_FOUR_AS:
-			if (c->dflag1 == COMMUNITY_NEIGHBOR_AS)
+			if ((c->flags >> 8 & 0xff) == COMMUNITY_NEIGHBOR_AS)
 				printf("neighbor-as:");
-			else if (c->dflag1 == COMMUNITY_LOCAL_AS)
+			else if ((c->flags >> 8 & 0xff) == COMMUNITY_LOCAL_AS)
 				printf("local-as:");
 			else
-				printf("%s:", log_as(c->c.e.data1));
+				printf("%s:", log_as(c->data1));
 			break;
 		case EXT_COMMUNITY_TRANS_IPV4:
-			addr.s_addr = htonl(c->c.e.data1);
+			addr.s_addr = htonl(c->data1);
 			printf("%s:", inet_ntoa(addr));
 			break;
 		}
 
-		switch (c->c.e.type) {
+		switch (type) {
 		case EXT_COMMUNITY_TRANS_TWO_AS:
 		case EXT_COMMUNITY_TRANS_FOUR_AS:
 		case EXT_COMMUNITY_TRANS_IPV4:
-			if (c->dflag2 == COMMUNITY_ANY)
+			if ((c->flags >> 16 & 0xff) == COMMUNITY_ANY)
 				printf("* ");
-			else if (c->dflag2 == COMMUNITY_NEIGHBOR_AS)
+			else if ((c->flags >> 16 & 0xff) ==
+			    COMMUNITY_NEIGHBOR_AS)
 				printf("neighbor-as ");
-			else if (c->dflag2 == COMMUNITY_LOCAL_AS)
+			else if ((c->flags >> 16 & 0xff) == COMMUNITY_LOCAL_AS)
 				printf("local-as ");
 			else
-				printf("%llu ",
-				    (unsigned long long)c->c.e.data2);
-			break;
-		case EXT_COMMUNITY_TRANS_OPAQUE:
-		case EXT_COMMUNITY_TRANS_EVPN:
-			printf("0x%llx ", (unsigned long long)c->c.e.data2);
+				printf("%u ", c->data2);
 			break;
 		case EXT_COMMUNITY_NON_TRANS_OPAQUE:
-			if (c->c.e.subtype == EXT_COMMUNITY_SUBTYPE_OVS) {
-				switch (c->c.e.data2) {
+			if (subtype == EXT_COMMUNITY_SUBTYPE_OVS) {
+				switch (c->data2) {
 				case EXT_COMMUNITY_OVS_VALID:
 					printf("valid ");
 					break;
@@ -259,10 +258,12 @@ print_community(struct filter_community *c)
 				}
 				break;
 			}
-			printf("0x%llx ", (unsigned long long)c->c.e.data2);
+			printf("0x%x%08x ", c->data1 & 0xffff, c->data2);
 			break;
+		case EXT_COMMUNITY_TRANS_OPAQUE:
+		case EXT_COMMUNITY_TRANS_EVPN:
 		default:
-			printf("0x%llx ", (unsigned long long)c->c.e.data2);
+			printf("0x%x%08x ", c->data1 & 0xffff, c->data2);
 			break;
 		}
 	}
@@ -385,9 +386,6 @@ print_mainconf(struct bgpd_config *conf)
 	if (conf->connectretry != INTERVAL_CONNECTRETRY)
 		printf("connect-retry %u\n", conf->connectretry);
 
-	if (conf->flags & BGPD_FLAG_NO_EVALUATE)
-		printf("route-collector yes\n");
-
 	if (conf->flags & BGPD_FLAG_DECISION_ROUTEAGE)
 		printf("rde route-age evaluate\n");
 
@@ -740,7 +738,7 @@ print_as(struct filter_rule *r)
 		printf("as-set \"%s\" ", r->match.as.name);
 		return;
 	}
-	switch(r->match.as.op) {
+	switch (r->match.as.op) {
 	case OP_RANGE:
 		printf("%s - ", log_as(r->match.as.as_min));
 		printf("%s ", log_as(r->match.as.as_max));
@@ -784,7 +782,7 @@ print_rule(struct bgpd_config *conf, struct filter_rule *r)
 		printf("eeeeeeeps. ");
 
 	if (r->peer.peerid) {
-		TAILQ_FOREACH(p, &conf->peers, entry)
+		RB_FOREACH(p, peer_head, &conf->peers)
 			if (p->conf.id == r->peer.peerid)
 				break;
 		if (p == NULL)
@@ -792,7 +790,7 @@ print_rule(struct bgpd_config *conf, struct filter_rule *r)
 		else
 			printf("%s ", log_addr(&p->conf.remote_addr));
 	} else if (r->peer.groupid) {
-		TAILQ_FOREACH(p, &conf->peers, entry)
+		RB_FOREACH(p, peer_head, &conf->peers)
 			if (p->conf.groupid == r->peer.groupid)
 				break;
 		if (p == NULL)
@@ -865,8 +863,8 @@ print_rule(struct bgpd_config *conf, struct filter_rule *r)
 	}
 
 	for (i = 0; i < MAX_COMM_MATCH; i++) {
-		struct filter_community *c = &r->match.community[i];
-		if (c->type != COMMUNITY_TYPE_NONE) {
+		struct community *c = &r->match.community[i];
+		if (c->flags != 0) {
 			printf("%s ", community_type(c));
 			print_community(c);
 		}
@@ -939,14 +937,14 @@ print_groups(struct bgpd_config *conf)
 	const char		 *c;
 
 	peer_cnt = 0;
-	TAILQ_FOREACH(p, &conf->peers, entry)
+	RB_FOREACH(p, peer_head, &conf->peers)
 		peer_cnt++;
 
 	if ((peerlist = calloc(peer_cnt, sizeof(struct peer_config *))) == NULL)
 		fatal("print_groups calloc");
 
 	i = 0;
-	TAILQ_FOREACH(p, &conf->peers, entry)
+	RB_FOREACH(p, peer_head, &conf->peers)
 		peerlist[i++] = &p->conf;
 
 	qsort(peerlist, peer_cnt, sizeof(struct peer_config *), peer_compare);
@@ -995,7 +993,7 @@ print_config(struct bgpd_config *conf, struct rib_names *rib_l)
 
 	print_mainconf(conf);
 	print_roa(&conf->roa);
-	print_as_sets(conf->as_sets);
+	print_as_sets(&conf->as_sets);
 	print_prefixsets(&conf->prefixsets);
 	print_originsets(&conf->originsets);
 	TAILQ_FOREACH(n, &conf->networks, entry)
