@@ -1,4 +1,4 @@
-/*	$OpenBSD: session.c,v 1.378 2019/04/07 10:52:30 claudio Exp $ */
+/*	$OpenBSD: session.c,v 1.395 2019/10/02 08:57:00 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004, 2005 Henning Brauer <henning@openbsd.org>
@@ -43,7 +43,6 @@
 #include <unistd.h>
 
 #include "bgpd.h"
-#include "mrt.h"
 #include "session.h"
 #include "log.h"
 
@@ -113,6 +112,14 @@ struct imsgbuf		*ibuf_main;
 struct mrt_head		 mrthead;
 time_t			 pauseaccept;
 
+static inline int
+peer_compare(const struct peer *a, const struct peer *b)
+{
+	return a->conf.id - b->conf.id;
+}
+
+RB_GENERATE(peer_head, peer, entry, peer_compare);
+
 void
 session_sighdlr(int sig)
 {
@@ -145,8 +152,8 @@ setup_listeners(u_int *la_cnt)
 			continue;
 		}
 
-		if (tcp_md5_listen(la->fd, &conf->peers) == -1)
-			fatal("tcp_md5_listen");
+		if (tcp_md5_prep_listener(la, &conf->peers) == -1)
+			fatal("tcp_md5_prep_listener");
 
 		/* set ttl to 255 so that ttl-security works */
 		if (la->sa.ss_family == AF_INET && setsockopt(la->fd,
@@ -191,6 +198,7 @@ session_main(int debug, int verbose)
 	struct ctl_conn		*ctl_conn;
 	struct listen_addr	*la;
 	void			*newp;
+	time_t			 now;
 	short			 events;
 
 	log_init(debug, LOG_DAEMON);
@@ -240,9 +248,7 @@ session_main(int debug, int verbose)
 	while (session_quit == 0) {
 		/* check for peers to be initialized or deleted */
 		if (!pending_reconf) {
-			for (p = TAILQ_FIRST(&conf->peers); p != NULL;
-			   p = next) {
-				next = TAILQ_NEXT(p, entry);
+			RB_FOREACH_SAFE(p, peer_head, &conf->peers, next) {
 				/* cloned peer that idled out? */
 				if (p->template && (p->state == STATE_IDLE ||
 				    p->state == STATE_ACTIVE) &&
@@ -267,10 +273,10 @@ session_main(int debug, int verbose)
 						session_demote(p, -1);
 					p->conf.demote_group[0] = 0;
 					session_stop(p, ERR_CEASE_PEER_UNCONF);
-					log_peer_warnx(&p->conf, "removed");
-					TAILQ_REMOVE(&conf->peers, p, entry);
 					timer_remove_all(p);
-					pfkey_remove(p);
+					tcp_md5_del_listener(conf, p);
+					log_peer_warnx(&p->conf, "removed");
+					RB_REMOVE(peer_head, &conf->peers, p);
 					free(p);
 					peer_cnt--;
 					continue;
@@ -358,12 +364,13 @@ session_main(int debug, int verbose)
 		idx_listeners = i;
 		timeout = 240;	/* loop every 240s at least */
 
-		TAILQ_FOREACH(p, &conf->peers, entry) {
+		now = getmonotime();
+		RB_FOREACH(p, peer_head, &conf->peers) {
 			time_t	nextaction;
 			struct peer_timer *pt;
 
 			/* check timers */
-			if ((pt = timer_nextisdue(p)) != NULL) {
+			if ((pt = timer_nextisdue(p, now)) != NULL) {
 				switch (pt->type) {
 				case Timer_Hold:
 					bgp_fsm(p, EVNT_TIMER_HOLDTIME);
@@ -378,18 +385,10 @@ session_main(int debug, int verbose)
 					bgp_fsm(p, EVNT_START);
 					break;
 				case Timer_IdleHoldReset:
-					p->IdleHoldTime /= 2;
-					if (p->IdleHoldTime <=
-					    INTERVAL_IDLE_HOLD_INITIAL) {
-						p->IdleHoldTime =
-						    INTERVAL_IDLE_HOLD_INITIAL;
-						timer_stop(p,
-						    Timer_IdleHoldReset);
-						p->errcnt = 0;
-					} else
-						timer_set(p,
-						    Timer_IdleHoldReset,
-						    p->IdleHoldTime);
+					p->IdleHoldTime =
+					    INTERVAL_IDLE_HOLD_INITIAL;
+					p->errcnt = 0;
+					timer_stop(p, Timer_IdleHoldReset);
 					break;
 				case Timer_CarpUndemote:
 					timer_stop(p, Timer_CarpUndemote);
@@ -405,7 +404,7 @@ session_main(int debug, int verbose)
 					fatalx("King Bula lost in time");
 				}
 			}
-			if ((nextaction = timer_nextduein(p)) != -1 &&
+			if ((nextaction = timer_nextduein(p, now)) != -1 &&
 			    nextaction < timeout)
 				timeout = nextaction;
 
@@ -414,7 +413,7 @@ session_main(int debug, int verbose)
 			if (p->wbuf.queued > 0 || p->state == STATE_CONNECT)
 				events |= POLLOUT;
 			/* is there still work to do? */
-			if (p->rpending)
+			if (p->rpending && p->rbuf && p->rbuf->wpos)
 				timeout = 0;
 
 			/* poll events */
@@ -502,7 +501,7 @@ session_main(int debug, int verbose)
 			session_dispatch_msg(&pfd[j],
 			    peer_l[j - idx_listeners]);
 
-		TAILQ_FOREACH(p, &conf->peers, entry)
+		RB_FOREACH(p, peer_head, &conf->peers)
 			if (p->rbuf && p->rbuf->wpos)
 				session_process_msg(p);
 
@@ -514,8 +513,8 @@ session_main(int debug, int verbose)
 			control_dispatch_msg(&pfd[j], &ctl_cnt, &conf->peers);
 	}
 
-	while ((p = TAILQ_FIRST(&conf->peers)) != NULL) {
-		TAILQ_REMOVE(&conf->peers, p, entry);
+	RB_FOREACH_SAFE(p, peer_head, &conf->peers, next) {
+		RB_REMOVE(peer_head, &conf->peers, p);
 		strlcpy(p->conf.shutcomm,
 		    "bgpd shutting down",
 		    sizeof(p->conf.shutcomm));
@@ -880,9 +879,11 @@ change_state(struct peer *peer, enum session_state state,
 		msgbuf_clear(&peer->wbuf);
 		free(peer->rbuf);
 		peer->rbuf = NULL;
+		peer->rpending = 0;
 		bzero(&peer->capa.peer, sizeof(peer->capa.peer));
-		imsg_compose(ibuf_main, IMSG_PFKEY_RELOAD, peer->conf.id,
-		    0, -1, NULL, 0);
+		if (!peer->template)
+			imsg_compose(ibuf_main, IMSG_PFKEY_RELOAD,
+			    peer->conf.id, 0, -1, NULL, 0);
 
 		if (event != EVNT_STOP) {
 			timer_set(peer, Timer_IdleHold, peer->IdleHoldTime);
@@ -927,8 +928,9 @@ change_state(struct peer *peer, enum session_state state,
 		}
 		break;
 	case STATE_ACTIVE:
-		imsg_compose(ibuf_main, IMSG_PFKEY_RELOAD, peer->conf.id,
-		    0, -1, NULL, 0);
+		if (!peer->template)
+			imsg_compose(ibuf_main, IMSG_PFKEY_RELOAD,
+			    peer->conf.id, 0, -1, NULL, 0);
 		break;
 	case STATE_OPENSENT:
 		break;
@@ -1109,8 +1111,10 @@ session_setup_socket(struct peer *p)
 		}
 
 		if (p->conf.ebgp) {
-			/* set TTL to foreign router's distance
-			   1=direct n=multihop with ttlsec, we always use 255 */
+			/*
+			 * set TTL to foreign router's distance
+			 * 1=direct n=multihop with ttlsec, we always use 255
+			 */
 			if (p->conf.ttlsec) {
 				ttl = 256 - p->conf.distance;
 				if (setsockopt(p->fd, IPPROTO_IP, IP_MINTTL,
@@ -1133,8 +1137,10 @@ session_setup_socket(struct peer *p)
 		break;
 	case AID_INET6:
 		if (p->conf.ebgp) {
-			/* set hoplimit to foreign router's distance
-			   1=direct n=multihop with ttlsec, we always use 255 */
+			/*
+			 * set hoplimit to foreign router's distance
+			 * 1=direct n=multihop with ttlsec, we always use 255
+			 */
 			if (p->conf.ttlsec) {
 #ifdef IPV6_MINHOPCOUNT
 				ttl = 256 - p->conf.distance;
@@ -2164,7 +2170,7 @@ parse_notification(struct peer *peer)
 	u_int8_t	 subcode;
 	u_int8_t	 capa_code;
 	u_int8_t	 capa_len;
-	u_int8_t	 shutcomm_len;
+	size_t		 shutcomm_len;
 	u_int8_t	 i;
 
 	/* just log */
@@ -2262,16 +2268,15 @@ parse_notification(struct peer *peer)
 	if (errcode == ERR_CEASE &&
 	    (subcode == ERR_CEASE_ADMIN_DOWN ||
 	     subcode == ERR_CEASE_ADMIN_RESET)) {
-		if (datalen >= sizeof(shutcomm_len)) {
-			memcpy(&shutcomm_len, p, sizeof(shutcomm_len));
-			p += sizeof(shutcomm_len);
-			datalen -= sizeof(shutcomm_len);
-			if(datalen < shutcomm_len) {
+		if (datalen > 1) {
+			shutcomm_len = *p++;
+			datalen--;
+			if (datalen < shutcomm_len) {
 			    log_peer_warnx(&peer->conf,
 				"received truncated shutdown reason");
 			    return (0);
 			}
-			if (shutcomm_len > (SHUT_COMM_LEN-1)) {
+			if (shutcomm_len > SHUT_COMM_LEN - 1) {
 			    log_peer_warnx(&peer->conf,
 				"received overly long shutdown reason");
 			    return (0);
@@ -2551,7 +2556,7 @@ session_dispatch_imsg(struct imsgbuf *ibuf, int idx, u_int *listener_cnt)
 				fatalx("reconf request not from parent");
 			nconf = new_config();
 
-			copy_config(nconf, imsg.data); 
+			copy_config(nconf, imsg.data);
 			pending_reconf = 1;
 			break;
 		case IMSG_RECONF_PEER:
@@ -2562,7 +2567,8 @@ session_dispatch_imsg(struct imsgbuf *ibuf, int idx, u_int *listener_cnt)
 			memcpy(&p->conf, imsg.data, sizeof(struct peer_config));
 			p->state = p->prev_state = STATE_NONE;
 			p->reconf_action = RECONF_REINIT;
-			TAILQ_INSERT_TAIL(&nconf->peers, p, entry);
+			if (RB_INSERT(peer_head, &nconf->peers, p) != NULL)
+				fatalx("%s: peer tree is corrupt", __func__);
 			break;
 		case IMSG_RECONF_LISTENER:
 			if (idx != PFD_PIPE_MAIN)
@@ -2675,7 +2681,7 @@ session_dispatch_imsg(struct imsgbuf *ibuf, int idx, u_int *listener_cnt)
 			kif = imsg.data;
 			depend_ok = kif->depend_state;
 
-			TAILQ_FOREACH(p, &conf->peers, entry)
+			RB_FOREACH(p, peer_head, &conf->peers)
 				if (!strcmp(p->conf.if_depend, kif->ifname)) {
 					if (depend_ok && !p->depend_ok) {
 						p->depend_ok = depend_ok;
@@ -2738,6 +2744,7 @@ session_dispatch_imsg(struct imsgbuf *ibuf, int idx, u_int *listener_cnt)
 			break;
 		case IMSG_CTL_SHOW_RIB:
 		case IMSG_CTL_SHOW_RIB_PREFIX:
+		case IMSG_CTL_SHOW_RIB_COMMUNITIES:
 		case IMSG_CTL_SHOW_RIB_ATTR:
 		case IMSG_CTL_SHOW_RIB_MEM:
 		case IMSG_CTL_SHOW_RIB_HASH:
@@ -2892,7 +2899,7 @@ getpeerbydesc(struct bgpd_config *c, const char *descr)
 	struct peer	*p, *res = NULL;
 	int		 match = 0;
 
-	TAILQ_FOREACH(p, &c->peers, entry)
+	RB_FOREACH(p, peer_head, &c->peers)
 		if (!strcmp(p->conf.descr, descr)) {
 			res = p;
 			match++;
@@ -2918,13 +2925,13 @@ getpeerbyip(struct bgpd_config *c, struct sockaddr *ip)
 	sa2addr(ip, &addr, NULL);
 
 	/* we might want a more effective way to find peers by IP */
-	TAILQ_FOREACH(p, &c->peers, entry)
+	RB_FOREACH(p, peer_head, &c->peers)
 		if (!p->conf.template &&
 		    !memcmp(&addr, &p->conf.remote_addr, sizeof(addr)))
 			return (p);
 
 	/* try template matching */
-	TAILQ_FOREACH(p, &c->peers, entry)
+	RB_FOREACH(p, peer_head, &c->peers)
 		if (p->conf.template &&
 		    p->conf.remote_addr.aid == addr.aid &&
 		    session_match_mask(p, &addr))
@@ -2937,11 +2944,8 @@ getpeerbyip(struct bgpd_config *c, struct sockaddr *ip)
 		if ((newpeer = malloc(sizeof(struct peer))) == NULL)
 			fatal(NULL);
 		memcpy(newpeer, loose, sizeof(struct peer));
-		for (id = UINT_MAX; id > UINT_MAX / 2; id--) {
-			TAILQ_FOREACH(p, &c->peers, entry)
-				if (p->conf.id == id)
-					break;
-			if (p == NULL)		/* we found a free id */
+		for (id = PEER_ID_DYN_MAX; id > PEER_ID_STATIC_MAX; id--) {
+			if (getpeerbyid(c, id) == NULL)	/* we found a free id */
 				break;
 		}
 		newpeer->template = loose;
@@ -2949,9 +2953,11 @@ getpeerbyip(struct bgpd_config *c, struct sockaddr *ip)
 		newpeer->state = newpeer->prev_state = STATE_NONE;
 		newpeer->reconf_action = RECONF_KEEP;
 		newpeer->rbuf = NULL;
+		newpeer->rpending = 0;
 		init_peer(newpeer);
 		bgp_fsm(newpeer, EVNT_START);
-		TAILQ_INSERT_TAIL(&c->peers, newpeer, entry);
+		if (RB_INSERT(peer_head, &c->peers, newpeer) != NULL)
+			fatalx("%s: peer tree is corrupt", __func__);
 		return (newpeer);
 	}
 
@@ -2961,13 +2967,11 @@ getpeerbyip(struct bgpd_config *c, struct sockaddr *ip)
 struct peer *
 getpeerbyid(struct bgpd_config *c, u_int32_t peerid)
 {
-	struct peer *p;
+	static struct peer lookup;
 
-	/* we might want a more effective way to find peers by id */
-	TAILQ_FOREACH(p, &c->peers, entry)
-		if (p->conf.id == peerid)
-			return (p);
-	return (NULL);
+	lookup.conf.id = peerid;
+
+	return RB_FIND(peer_head, &c->peers, &lookup);
 }
 
 int
@@ -3133,19 +3137,21 @@ void
 session_stop(struct peer *peer, u_int8_t subcode)
 {
 	char data[SHUT_COMM_LEN];
-	uint8_t datalen;
-	uint8_t shutcomm_len;
+	size_t datalen;
+	size_t shutcomm_len;
 	char *communication;
 
 	datalen = 0;
-
 	communication = peer->conf.shutcomm;
 
 	if ((subcode == ERR_CEASE_ADMIN_DOWN ||
 	    subcode == ERR_CEASE_ADMIN_RESET)
 	    && communication && *communication) {
 		shutcomm_len = strlen(communication);
-		if(shutcomm_len < SHUT_COMM_LEN) {
+		if (shutcomm_len > SHUT_COMM_LEN - 1) {
+		    log_peer_warnx(&peer->conf,
+			"trying to send overly long shutdown reason");
+		} else {
 			data[0] = shutcomm_len;
 			datalen = shutcomm_len + sizeof(data[0]);
 			memcpy(data + 1, communication, shutcomm_len);
@@ -3167,9 +3173,9 @@ session_stop(struct peer *peer, u_int8_t subcode)
 void
 merge_peers(struct bgpd_config *c, struct bgpd_config *nc)
 {
-	struct peer *p, *np;
+	struct peer *p, *np, *next;
 
-	TAILQ_FOREACH(p, &c->peers, entry) {
+	RB_FOREACH(p, peer_head, &c->peers) {
 		/* templates are handled specially */
 		if (p->template != NULL)
 			continue;
@@ -3179,8 +3185,15 @@ merge_peers(struct bgpd_config *c, struct bgpd_config *nc)
 			continue;
 		}
 
+		/* peer no longer uses TCP MD5SIG so deconfigure */
+		if (p->conf.auth.method == AUTH_MD5SIG &&
+		    np->conf.auth.method != AUTH_MD5SIG)
+			tcp_md5_del_listener(c, p);
+		else if (np->conf.auth.method == AUTH_MD5SIG)
+			tcp_md5_add_listener(c, np);
+
 		memcpy(&p->conf, &np->conf, sizeof(p->conf));
-		TAILQ_REMOVE(&nc->peers, np, entry);
+		RB_REMOVE(peer_head, &nc->peers, np);
 		free(np);
 
 		p->reconf_action = RECONF_KEEP;
@@ -3190,7 +3203,7 @@ merge_peers(struct bgpd_config *c, struct bgpd_config *nc)
 			session_demote(p, -1);
 
 		/* if session is not open then refresh pfkey data */
-		if (p->state < STATE_OPENSENT)
+		if (p->state < STATE_OPENSENT && !p->template)
 			imsg_compose(ibuf_main, IMSG_PFKEY_RELOAD,
 			    p->conf.id, 0, -1, NULL, 0);
 
@@ -3202,7 +3215,7 @@ merge_peers(struct bgpd_config *c, struct bgpd_config *nc)
 		/* apply the config to all clones of a template */
 		if (p->conf.template) {
 			struct peer *xp;
-			TAILQ_FOREACH(xp, &conf->peers, entry) {
+			RB_FOREACH(xp, peer_head, &c->peers) {
 				if (xp->template != p)
 					continue;
 				session_template_clone(xp, NULL, xp->conf.id,
@@ -3215,5 +3228,11 @@ merge_peers(struct bgpd_config *c, struct bgpd_config *nc)
 	}
 
 	/* pfkeys of new peers already loaded by the parent process */
-	TAILQ_CONCAT(&c->peers, &nc->peers, entry);
+	RB_FOREACH_SAFE(np, peer_head, &nc->peers, next) {
+		RB_REMOVE(peer_head, &nc->peers, np);
+		if (RB_INSERT(peer_head, &c->peers, np) != NULL)
+			fatalx("%s: peer tree is corrupt", __func__);
+		if (np->conf.auth.method == AUTH_MD5SIG)
+			tcp_md5_add_listener(c, np);
+	}
 }
